/******************************************************************************
*
* UserTextWindow   v 0 . 9 0    Beta 1
* Copyright  2004 David Randall
*
*******************************************************************************
*
* UserTextWindow creates fast scrollable text windows with word wrapping.
*
* UserTextWindow requires TIGCC 0.95 (0.94 ??) and Extgraphv v2.00 to compile.
* UserTextWindow requires user defined font files.
*
* UserTextWindow is meant to be an extension to the Extgraph API by providing
* very fast text output to the screen.
* 
* Thanks to the TIGCC and Extgraph teams. They have made programming our
* calculators fast, friendly, and fun.
*
* Send comments to drandall99@charter.net
*
******************************************************************************/



typedef struct
{
	unsigned char leftX;           // This is the left X position of the window
	unsigned char topY;            // This is the top Y position of the window
	unsigned char rightX;          // This is the right X position of the window
	unsigned char bottomY;         // This is the bottom Y position of the window
	unsigned char printLeftX;      // This is the left X position of the printable text area within the window
	unsigned char printTopY;       // This is the top Y position of the printable text area within the window
	unsigned char printRightX;     // This is the right X position of the printable text area within the window
	unsigned char printBottomY;    // This is the bottom Y position of the printable text area within the window
	unsigned char cursorX;         // This is the current X position of the cursor
	unsigned char cursorY;         // This is the current Y position of the cursor
	unsigned char lineSpacing;     // This is the number of additional pixels between lines of text
	unsigned char fontWidth;       // This is the width of the character sprite (limited to 8 pixels)
	unsigned char fontHeight;      // This is the height of the character sprite
	unsigned char maxCharsPerLine; // This is the maximum number of characters within the printable area
	unsigned char maxBufferLines;  // This is the maximum number of text lines within the allocated buffer
	unsigned char maxVisibleLines; // This is the maximum number of visible text lines
	unsigned char valid;           // This is a flag to indicate the window is valid
	unsigned char autoScroll;      // This is a flag to enable autoscrolling
	unsigned char wordWrap;        // This is a flag to enable word wrapping
	unsigned char scrollableLineIndex; // This is a pointer to the start of the scrollable text area
	unsigned char viewableLineIndex;   // This is a pointer to the top viewable line of text
	unsigned char writeLineIndex;  // This is a pointer to the buffer location
	unsigned short textColor;      // This is the color of the text
	unsigned short bkgdColor;      // This is the color of the text
	unsigned short printWidth;     // This is the width of the printable area
	unsigned short lineCount;      // This is the number of text lines added to the window
	unsigned short scrollLimit;    // This is a flag indicating the end of the scrollable area was reached
	long pCurrentPlane0;           // This is the address of the variable pointing to the current Plane0
	long pCurrentPlane1;           // This is the address of the variable pointing to the current Plane1
	char* pFont;                   // This is a pointer to the character sprites
	char* pBuffer;                 // This is a pointer to the text buffer
} UserTextWindowType;

// GraySingleSprite8_XOR_R is an optimized ASM routine that XORs the 
// single character sprite into both light and dark planes.
extern void GraySingleSprite8_XOR_R(
		register short x asm("%d0"),
		register short y asm("%d1"),
		register short h asm("%d2"),
		unsigned char *sprt0, // a2
		register void *dest0 asm("%a0"),
		register void *dest1 asm("%a1"))
		__attribute__((__stkparm__));

// UTW_Create defines the window and allocates memory for the text buffer
// If the user passes bad information, an invalid value a flag is set
void UTW_Create(
		unsigned char topLeftX, 
		unsigned char topLeftY, 
		unsigned char width, 
		unsigned char height, 
		unsigned char margin,
		unsigned char lineSpacing,
		unsigned char autoScroll,
		unsigned char wordWrap,
		unsigned short textColor,
		unsigned short bkgdColor,
		unsigned char* pFont, 
		unsigned char fontWidth, 
		unsigned char fontHeight,
		unsigned char bufferLines,
		unsigned long pCurrentPlane0,  // This user must pass in the address of the variable pointing to the current Plane0
		unsigned long pCurrentPlane1,  // This user must pass in the address of the variable pointing to the current Plane1
		UserTextWindowType* utw);
		
// UTW_Close sets the selected window as invalid and frees the allocated memory.
void UTW_Close(UserTextWindowType* utw);

// UTW_ClearHome clears the text buffer, clears the screen, sets the cursor to the top left 
// of the viewable area, and resets all read and write pointers to their initial value.
void UTW_ClearHome(UserTextWindowType* utw);

// UTW_ClearScreen clears the screen and sets the cursor to the top left of the viewable area.
void UTW_ClearScreen(UserTextWindowType* utw);

// UTW_Draw draws the selected window contents to the screen
void UTW_Draw(UserTextWindowType* utw);

// UTW_Print adds text to the selected window.
// If auto scroll is enabled, a the screen is scrolled up to keep the cursor position within the viewable area.
void UTW_Print(UserTextWindowType* utw, char* str);

// UTW_Scroll will scroll the selected window up or down by a user defined number of lines. 
// A negative value scrolls the window down and a positive value scrolls the window up. 
// A value of 1 is returned when the window reaches the end of the scrollable area.
short UTW_Scroll(UserTextWindowType* utw, signed char lines);
